/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.graph.impl;

import it.tidalwave.netbeans.visual.NodeScene;
import javax.annotation.Nonnull;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import org.netbeans.api.visual.widget.Scene;
import org.netbeans.api.visual.widget.Widget;
import org.netbeans.api.visual.action.ActionFactory;
import it.tidalwave.util.Id;
import it.tidalwave.semantic.graph.VisualSemanticController.EntityWidgetFactory;
import it.tidalwave.semantic.io.MIMETypes;
import it.tidalwave.semantic.io.SemanticUnmarshaller;
import it.tidalwave.semantic.io.SemanticUnmarshaller.Configuration;
import it.tidalwave.semantic.openrdf.OpenRDFSemanticFactory;
import it.tidalwave.semantic.other.Entity;
import it.tidalwave.semantic.other.Repository;
import it.tidalwave.semantic.other.SemanticFactory;
import it.tidalwave.semantic.other.Type;
import java.io.File;
import java.util.logging.Handler;
import java.util.logging.LogManager;
import java.util.logging.Logger;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.api.visual.graph.layout.GraphLayout;
import org.netbeans.api.visual.graph.layout.GraphLayoutFactory;
import org.netbeans.api.visual.graph.layout.GraphLayoutSupport;
import org.netbeans.api.visual.layout.LayoutFactory;
import org.netbeans.api.visual.layout.SceneLayout;
import org.openide.nodes.Node;

public class VisualSemanticTest 
  {
    private SemanticUnmarshaller unmarshaller;
    private NodeScene scene;
    private VisualSemanticControllerImpl controller;
    private Repository repository;
    private Configuration config;

    private final EntityWidgetFactory anonymousEntityWidgetFactory = new EntityWidgetFactory()
      {
        @Nonnull
        public Widget createWidget (@Nonnull Scene scene,
                                    @Nonnull final Node node)
          {
            final DefaultEntityWidget widget = new DefaultEntityWidget(scene, node);
            widget.setDisplayNameVisible(false);
            widget.getActions().addAction(ActionFactory.createMoveAction());
            return widget;
          }
      };

    @BeforeClass
    public static void setupLogging()
      {
        try
          {
            final File dir = new File(System.getProperty("basedir") + "/build/test/log");
            dir.mkdirs();
            System.err.println("LOGGING TO " + dir.getAbsolutePath());
            InputStream is = VisualSemanticTest.class.getResourceAsStream("log.properties");
            LogManager.getLogManager().readConfiguration(is);
            is.close();

            final SingleLineLogFormatter formatter = new SingleLineLogFormatter();
            Logger rootLogger = Logger.getLogger(VisualSemanticTest.class.getName());

            while (rootLogger.getParent() != null)
              {
                for (final Handler handler : rootLogger.getHandlers())
                  {
                    handler.setFormatter(formatter);
                  }

                rootLogger = rootLogger.getParent();
              }

            for (final Handler handler : rootLogger.getHandlers())
              {
                handler.setFormatter(formatter);
              }
          }
        catch (Exception e)
          {
            e.printStackTrace();
          }
      }


    @Before
    public void setup()
      {
        final SemanticFactory semanticFactory = OpenRDFSemanticFactory.Locator.findSemanticFactory();
        repository = semanticFactory.getDefaultRepository();
        unmarshaller = repository.getUnmarshaller();
        config = new Configuration();
        scene = new NodeScene();
        controller = new VisualSemanticControllerImpl(scene);
      }

    @Test
    public void dummy()
      {
      }

//    @Test
    public void testVRC()
      throws Exception
      {
        final String VRC_NS = "http://www.syr.edu/2008/12/28-vrc/vrc#";
        final Type TYPE_ACTIVITY = repository.findOrCreateType(new Id(VRC_NS + "activity"), "???");
        final Type TYPE_CHRONOLOGIC_LOCATION = repository.findOrCreateType(new Id(VRC_NS + "chronologicLocation"), "???");
        final Type TYPE_BIBLIOGRAPHIC_REFERENCE = repository.findOrCreateType(new Id(VRC_NS + "bibliographicReference"), "???");
        controller.registerWidgetFactory(TYPE_ACTIVITY, anonymousEntityWidgetFactory);
        controller.registerWidgetFactory(TYPE_CHRONOLOGIC_LOCATION, anonymousEntityWidgetFactory);
        controller.registerWidgetFactory(TYPE_BIBLIOGRAPHIC_REFERENCE, anonymousEntityWidgetFactory);

        run(new FileInputStream("/Users/fritz/VRC/RDF/QueryLeonardo.n3"), MIMETypes.N3);
//        run(new FileInputStream("/Users/fritz/VRC/RDF/build/test/QueryLeonardo.n3"), MIMETypes.N3);
      }

//    @Test FIXME
    public void testHenryStoryFOAF()
      throws Exception
      {
        run(getClass().getResourceAsStream("HenryStory.n3"), MIMETypes.N3);
      }

//    @Test
    public void testGeorgeWashingtonSimplified()
      throws Exception
      {
        run(getClass().getResourceAsStream("GeorgeWashingtonSimplified.rdf"), MIMETypes.RDF_XML);
      }

//    @Test
    public void testGeorgeWashington()
      throws Exception
      {
        run(getClass().getResourceAsStream("GeorgeWashington.rdf"), MIMETypes.RDF_XML);
      }

//    @Test
    public void testSimple()
      throws Exception
      {
        run(getClass().getResourceAsStream("Test1.n3"), MIMETypes.N3);
      }

//    @Test
    public void testRoyal92()
      throws Exception
      {
        final String NS_DAML = "http://www.daml.org/2001/01/gedcom/gedcom#";
        final Type TYPE_BIRTH = repository.findOrCreateType(new Id(NS_DAML + "Birth"), "Birth");
        final Type TYPE_DEATH = repository.findOrCreateType(new Id(NS_DAML + "Death"), "Death");
        final Type TYPE_MARRIAGE = repository.findOrCreateType(new Id(NS_DAML + "Marriage"), "Marriage");
        final Type TYPE_FAMILY = repository.findOrCreateType(new Id(NS_DAML + "Family"), "Family");
        controller.registerWidgetFactory(TYPE_BIRTH, anonymousEntityWidgetFactory);
        controller.registerWidgetFactory(TYPE_DEATH, anonymousEntityWidgetFactory);
        controller.registerWidgetFactory(TYPE_MARRIAGE, anonymousEntityWidgetFactory);
        controller.registerWidgetFactory(TYPE_FAMILY, anonymousEntityWidgetFactory);
        
        config.addDisplayNamePredicate(NS_DAML + "name");
        run(getClass().getResourceAsStream("royal92.daml"), MIMETypes.RDF_XML);
      }

    private void run (@Nonnull final InputStream is,
                      @Nonnull final String mimeType)
      throws Exception
      {
//        final InputStream is = getClass().getResourceAsStream("gedcom-facts.n3");
        final Reader reader = new InputStreamReader(is);
        unmarshaller.unmarshal(reader, mimeType, config);
        reader.close();
        Entity rootEntity = null;

        for (final Type type : repository.getAllTypes())
          {
            if (rootEntity == null)
              {
                rootEntity = type.getAllEntities().first();
              }

            controller.addEntities(type.getAllEntities());
          }

//        final SceneLayout devolveLayout = LayoutFactory.createDevolveWidgetLayout(mainLayer, LayoutFactory.createHorizontalFlowLayout (), true);
//        devolveLayout.invokeLayout();

          GraphLayout<Node, Node> graphLayout = GraphLayoutFactory.createTreeGraphLayout(100, 100, 50, 50, true);
          GraphLayoutSupport.setTreeGraphLayoutRootNode(graphLayout, controller.findNode(rootEntity));
          final SceneLayout sceneGraphLayout = LayoutFactory.createSceneGraphLayout(scene, graphLayout);
          sceneGraphLayout.invokeLayoutImmediately();
          
        SceneSupport.show(scene.createView());
          System.err.println("TYPES " + repository.getAllTypes());

        Thread.sleep(10000000);
      }

  }
